# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""A transform that turns grayscale images to RGB."""

import numpy
import torch
import torch.nn
import torchvision.transforms.functional


def square_center_pad(img: torch.Tensor) -> torch.Tensor:
    """Return a squared version of the image, centered on a canvas padded with
    zeros.

    Parameters
    ----------
    img
        The tensor to be transformed.  Expected to be in the form: ``[...,
        [1,3], H, W]`` (i.e. arbitrary number of leading dimensions).

    Returns
    -------
        Transformed tensor, guaranteed to be square (ie. equal height and
        width).
    """

    height, width = img.shape[-2:]
    maxdim = numpy.max([height, width])

    # padding
    left = (maxdim - width) // 2
    top = (maxdim - height) // 2
    right = maxdim - width - left
    bottom = maxdim - height - top

    return torchvision.transforms.functional.pad(
        img,
        [left, top, right, bottom],
        0,
        "constant",
    )


def grayscale_to_rgb(img: torch.Tensor) -> torch.Tensor:
    """Convert an image in grayscale to RGB.

    If the image is already in RGB format, then this is a NOOP - the same
    tensor is returned (no cloning).  If the image is in grayscale format
    (number of color channels = 1), then replicate it to obtain 3 color channels
    (a new copy is returned in this case).

    Parameters
    ----------
    img
        The tensor to be transformed.  Expected to be in the form: ``[...,
        [1,3], H, W]`` (i.e. arbitrary number of leading dimensions).

    Returns
    -------
    torch.Tensor
        Transformed tensor with 3 identical color channels.
    """

    if img.ndim < 3:
        raise TypeError(
            f"Input image tensor should have at least 3 dimensions, "
            f"but found {img.ndim}. If a grayscale image was provided, "
            f"ensure to include a channel dimension of size 1 ( i.e: "
            f"[1, height, width]).",
        )

    if img.shape[-3] not in (1, 3):
        raise TypeError(
            f"Input image tensor should have 1 or 3 color channels,"
            f"but found {img.shape[-3]}.",
        )

    if img.shape[-3] == 3:
        return img

    # it is a grayscale image - repeat the image 3 times
    repetitions = img.dim() * [1]
    repetitions[-3] = 3
    return img.repeat(*repetitions)


def rgb_to_grayscale(img: torch.Tensor) -> torch.Tensor:
    """Convert an image in RGB to grayscale.

    If the image is already in grayscale format, then this is a NOOP - the same
    tensor is returned (no cloning).  If the image is in RGB format
    (number of color channels = 3), then compresses the color channels into
    a single grayscale channel following this equation:

    .. math::

       grayscale = (0.2989 * r + 0.587 * g + 0.114 * b)

    A new tensor is returned in this case.

    Parameters
    ----------
    img
        The tensor to be transformed.  Expected to be in the form: ``[...,
        [1,3], H, W]`` (i.e. arbitrary number of leading dimensions).

    Returns
    -------
    torch.Tensor
        Transformed tensor with a single (grayscale) color channel.
    """

    if img.ndim < 3:
        raise TypeError(
            f"Input image tensor should have at least 3 dimensions, "
            f"but found {img.ndim}. If a grayscale image was provided, "
            f"ensure to include a channel dimension of size 1 ( i.e: "
            f"[1, height, width]).",
        )

    if img.shape[-3] not in (1, 3):
        raise TypeError(
            f"Input image tensor should have 1 or 3 planes,"
            f"but found {img.shape[-3]}",
        )

    if img.shape[-3] == 1:
        return img

    # it is an RGB image - use torchvision implementation
    return torchvision.transforms.functional.rgb_to_grayscale(img)


class SquareCenterPad(torch.nn.Module):
    """Transform to a squared version of the image, centered on a canvas padded
    with zeros.
    """

    def __init__(self):
        super().__init__()

    def forward(self, img: torch.Tensor) -> torch.Tensor:
        return square_center_pad(img)


class RGB(torch.nn.Module):
    """Wrapper class around :py:func:`.grayscale_to_rgb` to be used as a model
    transform.
    """

    def __init__(self):
        super().__init__()

    def forward(self, img: torch.Tensor) -> torch.Tensor:
        return grayscale_to_rgb(img)


class Grayscale(torch.nn.Module):
    """Wrapper class around :py:func:`rgb_to_grayscale` to be used as a model
    transform.
    """

    def __init__(self):
        super().__init__()

    def forward(self, img: torch.Tensor) -> torch.Tensor:
        return rgb_to_grayscale(img)
